%  TEST DESCRIPTION
%  Compare probability estimations obtained with {C-SVM + Platt} and our
%  proposed P-SVM solution, in one dimension. 
%  1) Create two normally distributed 1D datasets standing for class '1' and
%  class '-1'.
%  2) For each point, compute the probability to belong to class '1' (known from distributions).
%  3) Use one dataset as labels and the other as probabilities
%     (respectively used to train C-SVM and P-SVM)
%  4) Compare results in term of probability predictions and accuracy over a
%     test set.
%     We compare:
%       1- ytest_prob : true probability to belong to class '1'
%       2- ytest_prob_svm : Platt' estimations of the probabilities
%       3- ytest_prob_svme : SVM-P estimations

%% initialisation
addpath(genpath('.'))

clear all
close all
clc

seed=1987;
randn('seed',seed);
 
% classifier parameters
lambda = 1e-6;  
C = 100; Ct= 100;
kernel='gaussian';
kerneloption=1; % gamma of the RBF (SVM kernel)
lambda = 1e-6;  

% set confidence and compute a
epsilon=0.01;
confiance = 0.01;
nu = epsilon + confiance;
a = log(1/nu -1); 
plimit = nu ; 
%used as follow : let x be a point. 
%P(x=1) < plimit => x is of class '-1'; P(x=1) > 1- plimit => x is of class '1'; 

%% (training and test) data creation
nc = 200; 
ntest = 1000; 

% data set 1
sigma = 0.3; x01=[+0.5]; %0.3
% data set 2
sigma2 = 0.3; x02=[-0.5]; %0.3

% generate [X1,y1] (training data X1,training class y1 = {-1,1})
x1=sigma*randn(1,round(nc/2))+x01(1);
x2=sigma2*randn(1,round(nc/2))+x02(1);
X1=[x1 x2]';
X1=single(X1);
y1=[ones(1,round(nc/2)) -ones(1,round(nc/2))]';

% generate p1 (probability of X1 to belong to class '1')
pc1=exp(-sum((X1-repmat(x01,nc,1)).^2,2)/(2*sigma.^2));
pc2=exp(-sum((X1-repmat(x02,nc,1)).^2,2)/(2*sigma2.^2));
yp1=pc1./(pc1+pc2);
yp1=-(yp1<plimit)+(yp1>(1-plimit))+(yp1>plimit).*(yp1<(1-plimit)).*yp1; % proba + labeled data to avoid side effects

% label y1 (used to train C-SVM)
y1(yp1>0.5) = 1; y1(yp1<=0.5) = -1;

% generate [X2;y2] (test dataset)
x1=sigma*randn(1,round(ntest/2))+x01(1);
x2=sigma2*randn(1,round(ntest/2))+x02(1);
X2=[x1 x2]';
X2=single(X2);
y2=[ones(1,round(ntest/2)) -ones(1,round(ntest/2))]';

% generate p2
clear pc1 pc2;
pc1=exp(-sum((X2-repmat(x01,ntest,1)).^2,2)/(2*sigma.^2));
pc2=exp(-sum((X2-repmat(x02,ntest,1)).^2,2)/(2*sigma2.^2));
yp2=pc1./(pc1+pc2);
ytest_prob = yp2; % only probabilities
yp2=-(yp2<plimit)+(yp2>(1 - plimit))+(yp2>plimit).*(yp2<(1 - plimit)).*yp2; % proba + labeled data (to avoid side effects)

% label y2
y2(yp2>0.5) = 1; y2(yp2<=0.5) = -1;

Xapp=X1;Xtest=X2; 
yapp_svm=[y1]; % labeled data
yapp_svme=[yp1]; % proba + labeled data
ytest_lab=[y2]; % labeled data

%% Building a 2D Grid for function evaluation 
Xtest_grid = ([-1:.01:1]*(max([abs(x01(1)), abs(x02(1))]) + 3*max([sigma, sigma2])))';
nn = length(Xtest_grid);

%% probability over the grid
clear pc1 pc2;
nb_grid = length(Xtest_grid);
Xtest_grid_norm = Xtest_grid;

pc1=exp(-sum((Xtest_grid_norm-repmat(x01,nn,1)).^2,2)/(2*sigma.^2));
pc2=exp(-sum((Xtest_grid_norm-repmat(x02,nn,1)).^2,2)/(2*sigma2.^2));
ygrid_prob =pc1./(pc1+pc2);
ygrid_prob_2 =-(ygrid_prob<0.125)+(ygrid_prob>0.875)+(ygrid_prob>0.125).*(ygrid_prob<0.875).*ygrid_prob; % proba + labeled data to avoid side effects
ygrid_lab(ygrid_prob > 0.5) = 1; ygrid_lab(ygrid_prob <= 0.5) = -1;

%% classifier learning on a labeled problem

% training
[xsup,w,w0,pos,tps,alpha] = svmclass(Xapp,yapp_svm,C,lambda,kernel,kerneloption,1);
ypred_dist_svm = svmval(Xapp,xsup,w,w0,kernel,kerneloption,1);
ypred_lab_svm(ypred_dist_svm > 0,1) = 1; ypred_lab_svm(ypred_dist_svm <= 0,1) = -1;

% testing
ytest_dist_svm = svmval(Xtest,xsup,w,w0,kernel,kerneloption,[ones(length(Xtest),1)]);

% platt probabilities estimates
prior0 = sum(ypred_lab_svm == -1); prior1 = sum(ypred_lab_svm == 1);
%prior0 = 0.5; prior1 = 0.5;
[A,B]=platt(ypred_dist_svm,yapp_svm,prior0,prior1); 
ytest_prob_svm = 1./(1+exp(A*ytest_dist_svm+B));

% labels 
ytest_lab_svm(ytest_prob_svm > 0.5,1) = 1; ytest_lab_svm(ytest_prob_svm <= 0.5,1) = -1;

% %--------------  Evaluating the decision function on the grid
ygrid_dist_svm = svmval(Xtest_grid_norm,xsup,w,w0,kernel,kerneloption,[ones(length(Xtest_grid),1)]);
ygrid_prob_svm = 1./(1+exp(A*ygrid_dist_svm+B));


%% classifier learning with uncertainty on a semi labeled/probabilistic problem

% training
[xsup,w,w0,pos,tps,alpha] = usvmclass(Xapp,yapp_svme,C,Ct,epsilon,lambda,kernel,kerneloption,1); 
ypred_dist_svme = svmval(Xapp,xsup,w,w0,kernel,kerneloption,1);
ypred_lab_svme(ypred_dist_svm > 0,1) = 1; ypred_lab_svm(ypred_dist_svm <= 0,1) = -1;

% testing
ytest_dist_svme = svmval(Xtest,xsup,w,w0,kernel,kerneloption,[ones(length(Xtest),1)]);
ytest_prob_svme = (1./(1 + exp(-a*ytest_dist_svme)));

% labels
ytest_lab_svme(ytest_prob_svme > 0.5,1) = 1; ytest_lab_svme(ytest_prob_svme <= 0.5,1) = -1;

% %--------------  Evaluating the decision function on the grid
ygrid_dist_svme = svmval(Xtest_grid_norm,xsup,w,w0,kernel,kerneloption,[ones(length(Xtest_grid),1)]);
ygrid_prob_svme = (1./(1 + exp(-a*ygrid_dist_svme)));

%% errors estimation
ixp = logical((yp2>0)&(yp2<1)); % 0.125 < proba < 0.875
ixl = logical((yp2==-1)+(yp2==1)); % proba > 0.875 ou < 0.125 => label -1/1
 
% probability error
err_prob_svm = mean(abs(ytest_prob_svm - ytest_prob));
err_prob_svme = mean(abs(ytest_prob_svme - ytest_prob));
err_lab_svm = mean(ytest_lab_svm ~= ytest_lab);
err_lab_svme = mean(ytest_lab_svme ~= ytest_lab);

% accuracy
accuracy_svm = mean(ytest_lab_svm == ytest_lab);
accuracy_svme = mean(ytest_lab_svme == ytest_lab);

% alignement
align_lab_svm = 1 - sum(ytest_prob_svm.*ytest_prob)/(sqrt(sum(ytest_prob_svm.^2))*sqrt(sum(ytest_prob.^2)));
align_lab_svme = 1 - sum(ytest_prob_svme.*ytest_prob)/(sqrt(sum(ytest_prob_svme.^2))*sqrt(sum(ytest_prob.^2)));

% MISE
% mise_svm=sum((ytest_prob_svm-ytest_prob).^2./ytest_prob);
% mise_svme=sum((ytest_prob_svme-ytest_prob).^2./ytest_prob);

% Kullback Leibler
KullLeib_svm = sum(ytest_prob_svm.*log(ytest_prob_svm./ytest_prob)); 
KullLeib_svme = sum(ytest_prob_svme.*log(ytest_prob_svme./ytest_prob));


%% --------------- plotting true/ platt/svme probabilities estimates over the grid 
figure(1); 

ntest_grid = length(Xtest_grid);

subplot(2, 1, 1); 
plot(Xtest_grid, ygrid_prob, 'k', 'LineWidth', 1); hold on;
plot(Xtest_grid, ygrid_prob_svm, 'b', 'LineWidth', 1);
plot(Xtest_grid, ygrid_prob_svme, 'r',  'LineWidth', 1);
legend('P(y=1|x)', 'P(y=1|x)_{C-SVM + Platt}  ', 'P(y=1|x)_{P-SVM}'); 
xlim([-0.7 0.7])
ylabel('probability');
xlabel('x');

subplot(2, 1, 2); 
plot(Xtest_grid, abs(ygrid_prob - ygrid_prob_svm) , 'b', 'LineWidth', 1); hold on;
plot(Xtest_grid, abs(ygrid_prob - ygrid_prob_svme), 'r', 'LineWidth', 1);
legend('|P(y=1|x)_{true} - P(y=1|x)_{C-SVM + Platt}|', '|P(y=1|x)_{true} -  P(y=1|x)_{P-SVM}|');
xlim([-0.7 0.7])
ylabel('probability');
xlabel('x');


