function [xsup,w,d,pos,timeps,alpha,obj]=usvmclass(x,y,c,ct,epsilon,lambda,kernel,kerneloption,verbose,span, alphainit)
% USAGE [xsup,w,b,pos,timeps,alpha,obj]=usvmclass(x,y,c,lambda,kernel,kerneloption,verbose,span, alphainit)
%
% Support vector machine for CLASSIFICATION
% This routine classify the training set with a support vector machine
% using quadratic programming algorithm (active constraints method)
%
% INPUT
%
% Training set
%      x  		: input data 
%      y  		: output data
% parameters
%		c		: Bound on the lagrangian multipliers     
%		lambda		: Conditioning parameter for QP method
%		kernel		: kernel  type. classical kernel are
%
%		Name			parameters
%		'poly'		polynomial degree
%		'gaussian'	gaussian standard deviation
%
%		for more details see svmkernel
% 
%		kerneloption : parameters of kernel
%
%		for more details see svmkernel
%
% 		verbose : display outputs (default value is 0: no display)
%
%     Span    : span matrix for semiparametric learning 
%               This vector is sized Nbapp*Nbbasisfunction where
%               phi(i,j)= f_j(x(i));
%
%
%
% OUTPUT
%
% xsup	coordinates of the Support Vector
% w      weight
% b		bias
% pos    position of Support Vector
% timeps time for processing the scalar product
% alpha  Lagragian multiplier
% obj    Value of Objective function
%
%
% see also svmreg, svmkernel, svmval

%   17/11/10 R. Flamary E. Niaf - adding uncertainties handling
%	21/09/97 S. Canu
%	04/06/00 A. Rakotomamonjy   -inclusion of other kernel functions
%	04/05/01 S. Canu            -inclusion of multi-constraint optimization for frame-SVM
%
%       scanu@insa-rouen.fr, alain.rakoto@insa-rouen.fr


if nargin< 11
    alphainit=[];
end;

if nargin < 10 | isempty(span)
    A = y;
    b = 0;
else
    if span==1
        span=ones(size(y));
    end;
    [na,m]=size(span);
    [n un] = size(y);
    if n ~= na
        error('span, x and y  must have the same number of row')
    end
    A = (y*ones(1,m)).*span;
    b = zeros(m,1);
end
if nargin < 9
    verbose = 0;
end

if nargin < 8
    gamma = 1;
end

if nargin < 7
    kernel = 'gaussian';
end

if nargin < 6
    lambda = 0.000000001;
end

if nargin < 3
    c = inf;
end

lambda2 = 1e-5;

[n un] = size(y);

if ~isempty(x)
    [nl nc] = size(x);
    if n ~= nl
        error('x and y must have the same number of row')
    end
end;

if min(y) ~= -1
    error(' y must coded: 1 for class one and -1 for class two')
end

if verbose ~= 0 disp('building the distance matrix'); end;

ttt = cputime;

labeled=logical((y==-1)+(y==1));
probas=logical((y>0)&(y<1));
n=sum(labeled);
m=n+sum(probas);

ps  =  zeros(n,n);		
ps=svmkernel(x,kernel,kerneloption);

K1=ps(labeled,labeled);
K2=ps(probas,probas);
K3=ps(labeled,probas).*repmat(y(labeled),1,size(K2,1));

y2=[y(labeled);y(probas)];
x2=[x(labeled,:);x(probas,:)];

K1=K1.*(y(labeled)*y(labeled)');

G=[K1 -K3 K3;
    -K3' K2 -K2;
    K3' -K2 K2];

confiance = 0.025;
nu = epsilon + confiance;
a = log(1/nu -1);
zp=-log(1./(y(probas)+epsilon)-1)/a;
zn=-log(1./(y(probas)-epsilon)-1)/a;

A = [y(labeled) ;-ones(size(y(probas))) ;ones(size(y(probas)))];
b=0;
%----------------------------------------------------------------------
%      monqp(H,b,c) solves the quadratic programming problem:
% 
%    min 0.5*x'Hx - d'x   subject to:  A'x = b  and  0 <= x <= c 
%     x    
%----------------------------------------------------------------------
%H =ps.*(y*y'); 
e = [ones(size(y(labeled)));-zp;zn];
C= [c*ones(size(y(labeled)));ct*ones(size(y(probas))) ;ct*ones(size(y(probas)))];

timeps = cputime - ttt;

if verbose ~= 0 disp('in QP'); end;
if isinf(c)                                                           
    [alpha , lambda , pos] = monqpCinfty(G,e,A,b,lambda,verbose,x,ps,alphainit);  
else                                                                 
    [alpha , lambda , pos] = monqp(G,e,A,b,C,lambda,verbose,x,ps,alphainit);           
end
if verbose ~= 0 disp('out QP'); end;

alphaall=zeros(size(e));
alphaall(pos)=alpha;
obj=-0.5*alphaall'*G*alphaall +e'*alphaall;

idxalpha=find(pos<=n);
idxprobap=find((pos>n)&(pos<=m));
idxproban=find((pos>m));

pos(pos>m)=pos(pos>m)-m+n;

pos2=[pos(idxalpha);pos(idxprobap);pos(idxproban)];

if ~isempty(x)
    xsup = x2(pos2,:);
else
    xsup=[];
end;

ysup = y2(pos(idxalpha));

w = [(alpha((idxalpha)).*ysup);-(alpha(idxprobap));alpha(idxproban)];
d = lambda;

if verbose ~= 0  
    disp('max(alpha)') 
    fprintf(1,'%6.2f\n',max(alpha)) 
end 
